/*
 * Test.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Constants;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Plugins
{
    [ObjectId("f5813bfc-7fae-45bb-ab05-9e2d9f1ef49f")]
    [PluginFlags(
        PluginFlags.System | PluginFlags.Command |
        PluginFlags.Static | PluginFlags.Test)]
    internal sealed class Test : Default
    {
        #region Public Constructors
        public Test(
            IPluginData pluginData
            )
            : base(pluginData)
        {
            this.Flags |= AttributeOps.GetPluginFlags(GetType().BaseType) |
                AttributeOps.GetPluginFlags(this);
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region IState Members
        public override ReturnCode Initialize(
            Interpreter interpreter,
            IClientData clientData,
            ref Result result
            )
        {
            ReturnCode code;

            if (interpreter != null)
            {
                long token = 0;

                //
                // NOTE: Perform any plugin initialization steps here.
                //
                string name = FormatOps.PluginCommand(
                    this.Assembly, this.Name, typeof(_Commands.Nop), null);

                code = interpreter.AddCommand(
                    new _Commands.Nop(new CommandData(
                        name, null, null, clientData, typeof(_Commands.Nop).FullName,
                        CommandFlags.None, this, 0)), null, ref token, ref result);

                Version version = (code == ReturnCode.Ok) ? this.Version : null;

                if (code == ReturnCode.Ok)
                    code = interpreter.ProvidePackage(
                        this.GetType().FullName, version, ref result);

                if (code == ReturnCode.Ok)
                {
                    this.CommandTokens.Add(token);
                    this.Initialized = true;

                    result = StringList.MakeList(this.Name, version);
                }
            }
            else
            {
                result = "invalid interpreter";
                code = ReturnCode.Error;
            }

            return code;
        }

        ///////////////////////////////////////////////////////////////////////

        public override ReturnCode Terminate(
            Interpreter interpreter,
            IClientData clientData,
            ref Result result
            )
        {
            ReturnCode code;

            if (interpreter != null)
            {
                code = interpreter.RemoveCommands(
                    this, CommandFlags.None, clientData, ref result);

                Version version = (code == ReturnCode.Ok) ? this.Version : null;

                if (code == ReturnCode.Ok)
                    code = interpreter.WithdrawPackage(
                        this.GetType().FullName, version, ref result);

                if (code == ReturnCode.Ok)
                {
                    //
                    // HACK: Cleanup the test plugin token in the interpreter
                    //       state because this is the only place we can be
                    //       100% sure it will get done.
                    //
                    interpreter.testPluginToken = 0;

                    result = StringList.MakeList(this.Name, version);
                }
            }
            else
            {
                result = "invalid interpreter";
                code = ReturnCode.Error;
            }

            return code;
        }
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region IPlugin Members
        public override ReturnCode About(
            Interpreter interpreter,
            ref Result result
            )
        {
            result = FormatOps.PluginAbout(this, false);
            return ReturnCode.Ok;
        }

        ///////////////////////////////////////////////////////////////////////

        public override ReturnCode Options(
            Interpreter interpreter,
            ref Result result
            )
        {
            result = new StringList(DefineConstants.OptionList, false);
            return ReturnCode.Ok;
        }
        #endregion
    }
}
